/**
 * @fileOverview <p>The stompit jQuery plug-in offers some degree of frame-busting protection.</p>
 * <p>An event handler for window.onbeforeunload is registered to prompt the user to allow or deny any attempt to navigate away from the current page.
 * Any links (or other elements) that the stompit function is invoked on have an extra click handler set to remove the onbeforeunload handler.
 * The result is that clicking on any link designated as "safe" will result in normal behavior.
 * If any other process (such as a site loaded in an iframe that attempts frame-busting) initiates navigation away from the current page,
 * the user is prompted with a confirmation dialog.</p>
 *
 * <p>The function used as the click handler for "safe" elements can be accessed via:<br>
 * $.stompit.safeHandler<br>
 * You can invoke it at any time to effectively disable stompit:<br>
 * <code>$.stompit.safeHandler()</code><br>
 * Or you can use it as a handler for some other event that will result in page navigation that you want to be safe:<br>
 * <code>$( '#myElement' ).bind( 'myEvent', $.stompit.safeHandler );</code></p>
 * @author Mangala Sadhu Sangeet Singh Khalsa
 * @version 1.0
 */

/*global $, jQuery, safeNavHandler, frameBusterHandler */


( function($) {

/**
 * @name stompit
 * @function
 * @description Register the stompit event handler for the 'click' event for the specified links.<br>
 * stompit can register event handlers to all elements in the jQuery object it is invoked on at the
 * time of invocation, or it can use jQuery.delegate() to register a single event handler to a containing element.<br>
 * The links in the jQuery object the function is invoked on will be treated as "safe" - if they are clicked, they will be followed as normal.
 * Any other event that initiates navigation away from the current page will cause a dialog to pop up with this message:<br>
 * "Are you sure you want to close this page?"<br>
 * The buttons in the dialog are browser-dependent and cannot be changed.
 * @example
 * $( 'a' ).stompit();
 *
 * @param {String} [selector] jQuery selector for element(s) to apply stompit to <br>
 * If this parameter is supplied, stompit will use jQuery.delegate().  The event handler will be registered with the
 * jQuery object that stompit is invoked on, and applied to the elements within it that are matched by 'selector'.<br>
 * If this parameter is supplied, the 'options' parameter <em>must</em> be supplied - if you just want default behavior,
 * pass an empty object:<br>
 * <code>$( '#container' ).stompit( 'a', {} );</code>
 * @param {String|Function|Object} [options]
 * If a string is supplied, its value will be used as the message displayed in the navigation confirmation dialog.<br>
 * If a callback function is supplied, it should return a string whose value will be used as the message. If the function returns
 * an empty or falsy value, the link will be followed without prompting the user.<br>
 * If an object is supplied, it should have a 'message' property which can be a string or function to be used for the message.
 */
$.fn.stompit = function( selector, options ) {
	var defaultOptions = {
		message: 'Are you sure you want to close this page?'
	};

	try {
		if( $.isFunction( selector ) ) {
			defaultOptions.message = selector;
			selector = '';
		}
		else if( $.isPlainObject( selector ) ) {
			options = selector;
			selector = '';
		}

		if( options ) {
			$.extend( defaultOptions, options );
		}
		else if( selector ) {
			defaultOptions.message = selector;
			selector = '';
		}

		$.stompit = {
			settings: defaultOptions,
			isInitialized: false,
			safeHandler: safeNavHandler
		};

		if( !$.stompit.isInitialized ) {
			window.addEventListener( 'beforeunload', frameBusterHandler );
			$.stompit.isInitialized = true;
		}

		if( selector ) {
			this.delegate( selector, 'click', safeNavHandler );
		}
		else {
			this.click( safeNavHandler );
		}
	}
	catch( e ) {
		if( typeof console !== 'undefined' && typeof console.log === 'function' ) {
			console.log( '(stompit) ERROR:  ' + e.name + ':  ' + e.message );
		}
	}

	return this;
};

function safeNavHandler() {
	window.removeEventListener( 'beforeunload', frameBusterHandler );
	return true;
}

function frameBusterHandler( unloadEvent ) {
	var msg;

	if( typeof $.stompit.settings.message === 'function' ) {
		msg = $.stompit.settings.message();
	}
	else {
		msg = $.stompit.settings.message;
	}

	if( msg && msg.length ) {
		if( unloadEvent ) {
			unloadEvent.returnValue = msg;
		}
		return msg;
	}
}

}(jQuery));